@@ -267,7 +267,7 @@ For more, see `User Guide <user_guide.md>`_.
         if isinstance(self.rp_map, str):
             self._load_rp_map()
         
-        gene_loc_metadata = [(gene.chromosome, gene.start, gene.end, gene.annotation.get_name()) for gene in self.data_interface.gene_loc_set.regions]
+        gene_loc_metadata = [(gene.chromosome, gene.start, gene.end, gene.annotation.get_name(), gene.annotation.strand) for gene in self.data_interface.gene_loc_set.regions]
         regions_metadata = [r.to_tuple() for r in self.region_set.regions]
 
         return self.rp_map, gene_loc_metadata, regions_metadata
